import React, { useEffect, useRef, useMemo } from 'react';
import { scaleLinear, scaleBand } from 'd3';
import { visualizationColors } from 'config';

export type AxisComponentProps = {
  orientation: 'left' | 'bottom';
  type: 'numerical' | 'categorical';
  range: number[];
  domain: number[] //| string[];
};

const AxisComponent: React.FC<AxisComponentProps> = ({ orientation, type, range, domain }) => {
  const axisRef = useRef<SVGGElement>(null);
  const groupRef = useRef<SVGGElement>(null);

  const isVertical = orientation === 'left' // Check if it's a vertical axis

  const ticks = useMemo(() => {
    if (type === 'numerical') {
      const scale = scaleLinear()
        .domain(domain)
        .range(range);
      const width = range[1] - range[0];
      const pixelsPerTick = 30;
      const numberOfTicksTarget = Math.max(1, Math.floor(width / pixelsPerTick));
      
      return scale.ticks(numberOfTicksTarget).map(value => ({
        value,
        offset: scale(value),
      }));
    }
    return [];
  }, [type, domain, range, isVertical]);

  return (
    <svg>
      <g ref={groupRef}>
        <g ref={axisRef}>
        <path
          d={isVertical ? [
            "M", 6, range[0],
            "h", 6,
            "V", range[1],
            "h", -6,
          ].join(" ") : [
            "M", range[0], 6,
            "v", -6,
            "H", range[1],
            "v", 6,
          ].join(" ")}
          fill="none"
          stroke="currentColor"
        />
        {ticks.map(({ value, offset }) => (
          <g
            key={value}
            transform={`translate(${isVertical ? 12 : offset},${isVertical ? offset : 0})`}
          >
            <line
              transform={`rotate(${isVertical ? 0 : 45})`}
              x2={isVertical ? -6 : 6} // Adjust line length for vertical axis
              y2={isVertical ? 0 : 6}  // Adjust line length for horizontal axis
              stroke="currentColor"
            />
            <text
              style={{
                fontSize: "10px",
                textAnchor: isVertical ? "end" : "middle", // Align text according to orientation
                transform: isVertical ? "translateX(-10px) translateY(3px)" : "translateY(20px)",
              }}
            >
              {value}
            </text>
          </g>
        ))}
      </g>
      </g>
    </svg>
  );
};

export default AxisComponent;

//   useEffect(() => {
    
//     if (!axisRef.current || !groupRef.current) return;

//     const axisElement = select(axisRef.current);
//     const groupElement = select(groupRef.current);

//     const axisGenerator = orientation === 'left' ? axisLeft(scale) : axisBottom(scale);
//     if (ticks) axisGenerator.ticks(ticks);

//     axisGenerator
//       .tickSize(0)
//       .tickPadding(8)
//       .tickSizeOuter(4);

//     // Handle date-time formatting
//     if (type === 'datetime') {
//       axisGenerator.tickFormat((d) => {
//         const date = d as Date;
//         return date.toLocaleDateString(undefined, { month: 'short', day: 'numeric' }); // Customize this formatting as needed
//       });
//     }

//     axisElement.call(axisGenerator);

//     // Style axis elements
//     const AxisTextCol = visualizationColors.GPNeutral.colors[14];
//     groupElement.selectAll('.tick text').attr('class', 'font-mono').attr('fill', AxisTextCol[13]).style('stroke', 'none');

//     // Apply transformations based on the type and orientation
//     if (type === 'categorical') {
//       if (orientation === 'bottom') {
//         groupElement.selectAll('.tick text').attr('transform', 'rotate(-40) translate(0, 0)').style('text-anchor', 'end');
//       } else if (orientation === 'left') {
//         groupElement.selectAll('.tick text').attr('transform', 'rotate(0) translate(-2, -2)').style('text-anchor', 'end');
//       }
//     } else {
//       groupElement.selectAll('.tick text').attr('transform', 'rotate(0)').style('text-anchor', 'middle');
//     }

//   }, [scale, orientation, ticks, type]);

//   return (
//     <g ref={groupRef}>
//       <g ref={axisRef} />
//     </g>
//   );
// };

// export default AxisComponent;
